// sdxBaseEngine 2009 Sascha Bajus
// ----------------------------------------------------
//
// Der Shader erstellt eine einfache Beleuchtung die
// einen Gloss-Effect erzeugt.
// Der Shader hat 2 Technique-Sections um den Cullmode
// beim Rendern von Reflektionen anzupassen.
//
// ----------------------------------------------------

//Globals
matrix   World;
matrix   WorldViewProjection;
float3   LightDirection;
float    Brightness;
float4   Color;


//Vertex Output Structure
struct VS_OUTPUT {
   float4 pos				: POSITION;
   float3 normal			: TEXCOORD0;
   float3 lightdir			: TEXCOORD1;
};

//VertexShader Section
VS_OUTPUT vsout(float4 pos :POSITION , float3 normal :NORMAL, uniform bool bMirrored)
{
   VS_OUTPUT Out;
   Out.pos = mul(pos,WorldViewProjection);
   Out.normal = mul(normal,World);

		if (bMirrored == false)
    	{
   		 Out.lightdir= LightDirection;
   		 }
   		 else
    	{
    	Out.lightdir= LightDirection*-1;
   		 }
   		return Out;
}

//PixelShader Section
float4 psout(float3  normal :TEXCOORD0, float3 lightdir :TEXCOORD1) : COLOR
{
   float v =  Brightness + dot(normal,lightdir);
   return v * Color ;
}


//Technique Section
technique gloss
{
   pass pass0
   {
      cullmode=ccw;
      VertexShader = compile vs_1_1 vsout(false);
      PixelShader  = compile ps_2_0 psout();
   }
}

technique glossmirrored
{
   pass pass0
   {
      cullmode=cw;
      VertexShader = compile vs_1_1 vsout(true);
      PixelShader  = compile ps_2_0 psout();
   }
}
