﻿'# Title    : Simple LightShader Plasma Demo
'# Author   : Sascha Bajus
'# Created  : 06/2009
'# Copyright © 2009 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region

Imports SlimDX
Imports SlimDX.Direct3D9
Imports SlimDX.DirectInput
Imports sdxBaseEngine


Public Class Project : Inherits baseEngine


    Private _fx As Direct3D9.Effect
    Private _pass As Integer
    Private _Handle(4) As EffectHandle
    Private _Technique(1) As EffectHandle

    '#Shader Technique Enum
    Public Enum ShaderTechnique As Integer
        GlossLight = 0
        GlossLightMirrored = 1
    End Enum


    Private _usedot3 As Boolean
    Private _floor As sdxBillboard
    Private _torus As Mesh
    Private _plane As Plane
    Private _stencilmirror As sdxBillboard
    Private _tex As Texture


    Public Sub New()

        AddHandler dxRenderScene, AddressOf render
        AddHandler dxKeyboardState, AddressOf keyboard
        AddHandler dxMouseState, AddressOf mouse
        AddHandler dxLoadingProject, AddressOf loading
        '---------------------------------------------

        If Me.dxEngine("SlimDX - BaseEngine - Plasma Demo", , True) = False Then Me.Close()


    End Sub

#Region "loading"

    Private Sub loading()
        '#create Floor
        _floor = New sdxBillboard(New Vector3(12, 0, 12))
        _floor.addTexture("floor.png", Color.Magenta)


        '#create plane for reflection
        _plane = New Plane(New Vector3(0, 0, 0), New Vector3(0, 1, 0))
        '#dummy texture to render to stencilbuffer
        _stencilmirror = New sdxBillboard(New Vector3(5, 0, 5))
        _stencilmirror.addTexture("dummy.png", Color.Magenta)

        '#create torus
        _torus = Mesh.CreateTorus(dxDevice, 0.6, 1.2, 44, 44)

        '#enable Zbuffer
        dxZBuffer.Enabled = True


        '#create Camset for automatic Camera rotation
        Dim camset As New engineCamera.cameraSetRotate
        With camset
            .Angle = 0.02
            .Height = 1
            .maxHeight = 12
            .Radius = 12
            .RotationSpeed = 1900
            .Curved = True
            .CurvedSpeed = 800
        End With
        dxCamera.useCameraSet(camset)



        '#setup ParticleEngine for PlasmaEffect
        With dxEffect.ParticleEngine
            .resetEngine()
            .Texture = dxTexture.loadTexture("particle.png", Color.Black)
            .StartColor = Color.FromArgb(255, 64, 64, 175)
            .EndColor = Color.FromArgb(255, 175, 64, 33)
            .AdjustSaturation = 1.2
            .ColorFading = True
            .ColorInterpolation = True
            .ColorRandom = False
            .ParticleCount = 200
            .DecreaseLifetime(0.04) = 0.015
            .Emitter = New Vector3(0, 0, 0)
            .EmitterRandom = New Vector3(2.5, 0, 0)
            .ParticleSize = 0.09
            .Direction(-0.0011) = New Vector3(0, 0.15, 0)
            .DirectionRandom = New Vector3(0.08, 0.1, 0)
            .DirectionBothSides = True
            .RenderSpeed = 950
            .RenderDoubled = True
        End With



        '#load Shader fx-file
        Try
            _fx = Direct3D9.Effect.FromFile(dxDevice, dxEngine.ApplicationPath & "\shaderGlossLight.fx", ShaderFlags.PartialPrecision)
        Catch ex As Exception
            MsgBox("Error Loading FX EffectResource ' shaderGlossLight.fx '", MsgBoxStyle.Critical, "FX File Error")
        End Try

        '#Shader Parameter Handles
        _Handle(0) = _fx.GetParameter(Nothing, "WorldViewProjection")
        _Handle(1) = _fx.GetParameter(Nothing, "LightDirection")
        _Handle(2) = _fx.GetParameter(Nothing, "Color")
        _Handle(3) = _fx.GetParameter(Nothing, "Brightness")
        _Handle(4) = _fx.GetParameter(Nothing, "World")


        '#Shader Technique Handles
        _Technique(0) = _fx.GetTechnique("gloss")
        _Technique(1) = _fx.GetTechnique("glossmirrored")


        '#Shader Property
        fxGlossColor = New Color4(1, 0.88, 0.88)
        fxBrightness = 0.5
        fxLightDirection = New Vector3(0, -1, 0)

    End Sub
#End Region
#Region "Shader Property"
    ''' <summary>
    ''' Legt die Worldviewprojection Matrix fest.
    ''' </summary>
    Public WriteOnly Property fxWorldViewProjection() As Matrix
        Set(ByVal value As Matrix)
            _fx.SetValue(_Handle(0), value)
        End Set
    End Property
    ''' <summary>
    ''' Legt die Lichtrichtung fest.
    ''' </summary>
    Public WriteOnly Property fxLightDirection() As Vector3
        Set(ByVal value As Vector3)
            _fx.SetValue(_Handle(1), value)
        End Set
    End Property
    ''' <summary>
    ''' Legt die Glossfarbe fest.
    ''' </summary>
    ''' <value></value>
    ''' <remarks></remarks>
    Public WriteOnly Property fxGlossColor() As Color4
        Set(ByVal value As Color4)
            _fx.SetValue(_Handle(2), value)
        End Set
    End Property
    ''' <summary>
    ''' Legt die Helligkeit fest
    ''' </summary>
    Public WriteOnly Property fxBrightness() As Single
        Set(ByVal value As Single)
            _fx.SetValue(_Handle(3), value)
        End Set
    End Property
    ''' <summary>
    ''' Legt die Worldmatrix fest
    ''' </summary>
    Public WriteOnly Property fxWorld() As Matrix
        Set(ByVal value As Matrix)
            _fx.SetValue(_Handle(4), value)
        End Set
    End Property

    ''' <summary>
    ''' Legt die Technik des Shaders fest.
    ''' </summary>
    Public WriteOnly Property fxTechnique() As ShaderTechnique
        Set(ByVal value As ShaderTechnique)
            _fx.Technique = _Technique(value)
        End Set
    End Property
#End Region
#Region "## render ##"

    Private Sub render()

        '// Setup for StencilBuffer writing "1"
        dxStencilBuffer.StencilAllwaysOne = True
        dxZBuffer.WriteEnabled = False
        dxEngine.BackBufferDraw = False

        '// Draw reflection area to StencilBuffer
        dxTransformWorld = Matrix.Translation(New Vector3(0.2, 0, -0.2))
        _stencilmirror.draw()


        '// Setup StencilBuffer writing Pixel to Mask
        dxStencilBuffer.StencilKeepIfOne = True
        dxZBuffer.WriteEnabled = True
        dxEngine.BackBufferDraw = True


        '// Draw reflected Torus - ShaderTechnique "gloss-mirrored"
        dxTransformWorld = dxMatrix.TimebasedRotationXYZ(New Vector3(0.1, 0.1, 0), 800) * Matrix.Translation(New Vector3(0, 2.5, 0)) * Matrix.Reflection(_plane)

        fxTechnique = ShaderTechnique.GlossLightMirrored
        fxWorldViewProjection = dxWorld.WorldViewProjection
        fxWorld = dxWorld.World

        _fx.Begin()
        _fx.BeginPass(0)
        _torus.DrawSubset(0)
        _fx.EndPass()
        _fx.End()


        '// Draw reflected Particle
        dxTransformWorld = Matrix.Identity * Matrix.Reflection(_plane)
        dxZBuffer.WriteEnabled = False
        dxTexture.AlphaBlending3 = True
        dxEffect.ParticleEngine.Draw()
        dxTexture.AlphaBlending3 = False
        dxZBuffer.WriteEnabled = True

        '// clear Stencilbuffer and Disable 
        dxStencilBuffer.clear()
        dxStencilBuffer.Enabled = False


        '// Draw  Torus - ShaderTechnique "gloss"
        dxTransformWorld = dxMatrix.TimebasedRotationXYZ(New Vector3(0.1, 0.1, 0), 800) * Matrix.Translation(New Vector3(0, 2.5, 0))

        fxTechnique = ShaderTechnique.GlossLight
        fxWorldViewProjection = dxWorld.WorldViewProjection
        fxWorld = dxWorld.World

        _fx.Begin()
        _fx.BeginPass(0)
        _torus.DrawSubset(0)
        _fx.EndPass()
        _fx.End()


        '// Draw  Particle
        dxTransformWorld = Matrix.Identity
        dxZBuffer.WriteEnabled = False
        dxTexture.AlphaBlending3 = True
        dxEffect.ParticleEngine.Draw()
        dxZBuffer.WriteEnabled = True

        '// Draw Floor
        _floor.setTexture(0, 0)
        dxTexture.AlphaBlending3 = True
        dxTransformWorld = Matrix.Identity
        _floor.draw()
        dxTexture.AlphaBlending3 = False

        '// render FPS
        dx2DText.drawFPS()
    End Sub
#End Region
#Region "## Inputdevices ##"
    Private Sub keyboard()
        If dxEngine.KeyPressed(Key.Escape) Then Me.Close()
        If dxEngine.KeyAsSwitch(Key.Space) Then _usedot3 = Not _usedot3
    End Sub

    Private Sub mouse()
    End Sub
#End Region
#Region "## Exit Application and Cleanup##"

    Private Sub closeForm() Handles Me.FormClosing
        closeApplication()
    End Sub

    Private Sub closeApplication()
        If Not _floor Is Nothing Then _floor.dispose()
        If Not _torus Is Nothing Then _torus.Dispose()
        If Not _stencilmirror Is Nothing Then _stencilmirror.dispose()
        If Not _tex Is Nothing Then _tex.Dispose()
        If Not _fx Is Nothing Then _fx.Dispose()
        dxEngine.closeBaseEngine()
        End
    End Sub
#End Region
End Class

